@@ -14,7 +14,7 @@ from .connection import ConnectionPool
 from .message import Packet, Message
 from .config import CURRENT_CONFIG
 
-LOGGER = logging.getLogger(__name__)
+LOGGER = logging.getLogger('quickmq')
 
 
 class AmqpSession:
@@ -77,7 +77,7 @@ class AmqpSession:
                 key, msg = val
             else:
                 msg = val
-            self.publish(msg, key, exchange=exchange, block=confirm_delivery)
+            self.publish(msg, key, exchange=exchange, confirm_delivery=confirm_delivery)
 
     def disconnect(self, *args) -> None:
         if not args:
